@@ -12,7 +12,7 @@ use feature ();
 package Zydeco;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.513';
+our $VERSION   = '0.514';
 
 use Keyword::Simple ();
 use PPR;
@@ -248,13 +248,18 @@ our $GRAMMAR = qr{
 			(?&MxpTypeSpec)|(?&PerlBlock)
 		)#</MxpExtendedTypeSpec>
 		
+		(?<MxpSignatureVariable>
+			[\$\@\%]
+			(?&PerlIdentifier)
+		)#</MxpSignatureVariable>
+		
 		(?<MxpSignatureElement>
 		
 			(?&PerlOWS)
 			(?: (?&MxpExtendedTypeSpec))?                 # CAPTURE:type
 			(?&PerlOWS)
 			(?:                                           # CAPTURE:name
-				(?&PerlVariable) | (\*(?&PerlIdentifier))
+				(?&MxpSignatureVariable) | (\*(?&PerlIdentifier) | [\$\@\%] )
 			)
 			(?:                                           # CAPTURE:postamble
 				\? | ((?&PerlOWS)=(?&PerlOWS)(?&PerlScalarExpression))
@@ -767,7 +772,16 @@ sub _handle_signature_list {
 			$sig =~ s/^\*\Q$name//xs;
 			$sig =~ s/^((?&PerlOWS)) $GRAMMAR//xso;
 		}
-		elsif ($sig =~ /^((?&PerlVariable)) $GRAMMAR/xso) {
+		elsif ($sig =~ /^ ( [\$\@\%] ) (?: [=),?] | (?&PerlNWS) | $ ) $GRAMMAR/xso) {
+			state $dummy = 0;
+			my $name = substr($sig,0,1) . '____ZYDECO_DUMMY_VAR_' . ++$dummy;
+			$parsed[-1]{name}       = $name;
+			$parsed[-1]{named}      = 0;
+			$parsed[-1]{positional} = 1;
+			$sig = substr($sig, 1);
+			$sig =~ s/^((?&PerlOWS)) $GRAMMAR//xs;
+		}
+		elsif ($sig =~ /^((?&MxpSignatureVariable)) $GRAMMAR/xso) {
 			my $name = $1;
 			$parsed[-1]{name}       = $name;
 			$parsed[-1]{named}      = 0;
@@ -947,6 +961,11 @@ sub _handle_role_list {
 	return join(",", @return);
 }
 
+sub _stringify_attributes {
+	my @quoted = map sprintf(q("%s"), quotemeta(substr $_, 1)), @{ $_[1] || [] };
+	sprintf '[%s]', join q[,], @quoted;
+}
+
 sub _handle_name_list {
 	my ($me, $names) = @_;
 	return unless $names;
@@ -980,9 +999,10 @@ sub _handle_factory_keyword {
 	if (!$has_sig) {
 		my $munged_code = sprintf('sub { my ($factory, $class) = (@_); do %s }', $code);
 		return sprintf(
-			'q[%s]->_factory(%s, { caller => __PACKAGE__, code => %s, optimize => %d });',
+			'q[%s]->_factory(%s, { attributes => %s, caller => __PACKAGE__, code => %s, optimize => %d });',
 			$me,
 			($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
+			$me->_stringify_attributes($attrs),
 			$optim ? B::perlstring($munged_code) : $munged_code,
 			!!$optim,
 		);
@@ -990,9 +1010,10 @@ sub _handle_factory_keyword {
 	my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $me->_handle_signature_list($sig);
 	my $munged_code = sprintf('sub { my($factory,$class,%s)=(shift,shift,@_); %s; do %s }', $signature_var_list, $extra, $code);
 	sprintf(
-		'q[%s]->_factory(%s, { caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
+		'q[%s]->_factory(%s, { attributes => %s, caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
 		$me,
 		($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
+		$me->_stringify_attributes($attrs),
 		$optim ? B::perlstring($munged_code) : $munged_code,
 		!!$signature_is_named,
 		$type_params_stuff,
@@ -1026,9 +1047,10 @@ sub _handle_method_keyword {
 			my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $me->_handle_signature_list($sig);
 			my $munged_code = sprintf('sub { my($self,%s)=(shift,@_); %s; my $class = ref($self)||$self; do %s }', $signature_var_list, $extra, $code);
 			$return = sprintf(
-				'q[%s]->_can(%s, { caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
+				'q[%s]->_can(%s, { attributes => %s, caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
 				$me,
 				($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
+				$me->_stringify_attributes($attrs),
 				$optim ? B::perlstring($munged_code) : $munged_code,
 				!!$signature_is_named,
 				$type_params_stuff,
@@ -1038,9 +1060,10 @@ sub _handle_method_keyword {
 		else {
 			my $munged_code = sprintf('sub { my $self = $_[0]; my $class = ref($self)||$self; do %s }', $code);
 			$return = sprintf(
-				'q[%s]->_can(%s, { caller => __PACKAGE__, code => %s, optimize => %d });',
+				'q[%s]->_can(%s, { attributes => %s, caller => __PACKAGE__, code => %s, optimize => %d });',
 				$me,
 				($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
+				$me->_stringify_attributes($attrs),
 				$optim ? B::perlstring($munged_code) : $munged_code,
 				!!$optim,
 			);
@@ -1051,8 +1074,9 @@ sub _handle_method_keyword {
 			my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $me->_handle_signature_list($sig);
 			my $munged_code = sprintf('sub { my($self,%s)=(shift,@_); %s; my $class = ref($self)||$self; do %s }', $signature_var_list, $extra, $code);
 			$return = sprintf(
-				'q[%s]->wrap_coderef({ caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
+				'q[%s]->wrap_coderef({ attributes => %s, caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
 				'MooX::Press',
+				$me->_stringify_attributes($attrs),
 				$optim ? B::perlstring($munged_code) : $munged_code,
 				!!$signature_is_named,
 				$type_params_stuff,
@@ -1062,8 +1086,9 @@ sub _handle_method_keyword {
 		else {
 			my $munged_code = sprintf('sub { my $self = $_[0]; my $class = ref($self)||$self; do %s }', $code);
 			$return = sprintf(
-				'q[%s]->wrap_coderef({ caller => __PACKAGE__, code => %s, optimize => %d });',
+				'q[%s]->wrap_coderef({ attributes => %s, caller => __PACKAGE__, code => %s, optimize => %d });',
 				'MooX::Press',
+				$me->_stringify_attributes($attrs),
 				$optim ? B::perlstring($munged_code) : $munged_code,
 				!!$optim,
 			);
@@ -1099,9 +1124,10 @@ sub _handle_multimethod_keyword {
 		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $me->_handle_signature_list($sig);
 		my $munged_code = sprintf('sub { my($self,%s)=(shift,@_); %s; my $class = ref($self)||$self; do %s }', $signature_var_list, $extra, $code);
 		return sprintf(
-			'q[%s]->_multimethod(%s, { caller => __PACKAGE__, code => %s, named => %d, signature => %s, %s });',
+			'q[%s]->_multimethod(%s, { attributes => %s, caller => __PACKAGE__, code => %s, named => %d, signature => %s, %s });',
 			$me,
 			($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
+			$me->_stringify_attributes($attrs),
 			$munged_code,
 			!!$signature_is_named,
 			$type_params_stuff,
@@ -1111,9 +1137,10 @@ sub _handle_multimethod_keyword {
 	else {
 		my $munged_code = sprintf('sub { my $self = $_[0]; my $class = ref($self)||$self; do %s }', $code);
 		return sprintf(
-			'q[%s]->_multimethod(%s, { caller => __PACKAGE__, code => %s, named => 0, signature => sub { @_ }, %s });',
+			'q[%s]->_multimethod(%s, { attributes => %s, caller => __PACKAGE__, code => %s, named => 0, signature => sub { @_ }, %s });',
 			$me,
 			($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
+			$me->_stringify_attributes($attrs),
 			$munged_code,
 			$extra_code,
 		);
@@ -1152,10 +1179,11 @@ sub _handle_modifier_keyword {
 			$munged_code = sprintf('sub { my($self,%s)=(shift,@_); %s; my $class = ref($self)||$self; do %s }', $signature_var_list, $extra, $code);
 		}
 		sprintf(
-			'q[%s]->_modifier(q(%s), %s, { caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
+			'q[%s]->_modifier(q(%s), %s, { attributes => %s, caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
 			$me,
 			$kind,
 			$processed_names,
+			$me->_stringify_attributes($attrs),
 			$optim ? B::perlstring($munged_code) : $munged_code,
 			!!$signature_is_named,
 			$type_params_stuff,
@@ -1165,10 +1193,11 @@ sub _handle_modifier_keyword {
 	elsif ($kind eq 'around') {
 		my $munged_code = sprintf('sub { my ($next, $self) = @_; my $class = ref($self)||$self; do %s }', $code);
 		sprintf(
-			'q[%s]->_modifier(q(%s), %s, { caller => __PACKAGE__, code => %s, optimize => %d });',
+			'q[%s]->_modifier(q(%s), %s, { attributes => %s, caller => __PACKAGE__, code => %s, optimize => %d });',
 			$me,
 			$kind,
 			$processed_names,
+			$me->_stringify_attributes($attrs),
 			$optim ? B::perlstring($munged_code) : $munged_code,
 			!!$optim,
 		);
@@ -1176,10 +1205,11 @@ sub _handle_modifier_keyword {
 	else {
 		my $munged_code = sprintf('sub { my $self = $_[0]; my $class = ref($self)||$self; do %s }', $code);
 		sprintf(
-			'q[%s]->_modifier(q(%s), %s, { caller => __PACKAGE__, code => %s, optimize => %d });',
+			'q[%s]->_modifier(q(%s), %s, { attributes => %s, caller => __PACKAGE__, code => %s, optimize => %d });',
 			$me,
 			$kind,
 			$processed_names,
+			$me->_stringify_attributes($attrs),
 			$optim ? B::perlstring($munged_code) : $munged_code,
 			!!$optim,
 		);
@@ -2206,7 +2236,7 @@ sub _include {
 #{
 #	package Zydeco::Anonymous::Package;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.513';
+#	our $VERSION   = '0.514';
 #	use overload q[""] => sub { ${$_[0]} }, fallback => 1;
 #	sub DESTROY {}
 #	sub AUTOLOAD {
@@ -2217,7 +2247,7 @@ sub _include {
 #	
 #	package Zydeco::Anonymous::Class;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.513';
+#	our $VERSION   = '0.514';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub new {
 #		my $me = shift;
@@ -2230,12 +2260,12 @@ sub _include {
 #	
 #	package Zydeco::Anonymous::Role;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.513';
+#	our $VERSION   = '0.514';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	
 #	package Zydeco::Anonymous::ParameterizableClass;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.513';
+#	our $VERSION   = '0.514';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
@@ -2249,7 +2279,7 @@ sub _include {
 #
 #	package Zydeco::Anonymous::ParameterizableRole;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.513';
+#	our $VERSION   = '0.514';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
